@@ -1,16 +1,18 @@
-pub(crate) struct SelectIndicesBase<'a, T>
+pub(crate) struct SelectIndicesBase<'a, T, I>
 {
     pub(crate) data: T,
-    pub(crate) indices: &'a [usize],
+    pub(crate) indices: &'a [I],
     pub(crate) start: usize,
     pub(crate) end: usize,
 }
 
+use num_traits::{ PrimInt, ToPrimitive };
+
 use std::collections::HashSet;
 
-impl<T> SelectIndicesBase<'_, &[T]>
+impl<T, I: Copy + Clone + PrimInt + ToPrimitive> SelectIndicesBase<'_, &[T], I>
 {
-    pub(crate) fn safety_check(slice: &[T], indices: &[usize])
+    pub(crate) fn safety_check(slice: &[T], indices: &[I])
     {
         let len = slice.len();
         let indices_len = indices.len();
@@ -23,15 +25,17 @@ impl<T> SelectIndicesBase<'_, &[T]>
         // TODO: Safety checks without heap allocation
         
         indices.iter().for_each(|&i| {
+            let i = i.to_usize()
+                .expect("select_indices was given indices that cannot be converted to usize!");
             assert!(i < len);
             assert!(indexset.insert(i));
         });
     }
 }
 
-impl<T> SelectIndicesBase<'_, &mut [T]>
+impl<T, I: Copy + Clone + PrimInt + ToPrimitive> SelectIndicesBase<'_, &mut [T], I>
 {
-    pub(crate) fn safety_check(slice: &mut [T], indices: &[usize])
+    pub(crate) fn safety_check(slice: &mut [T], indices: &[I])
     {
         let len = slice.len();
         let indices_len = indices.len();
@@ -44,13 +48,15 @@ impl<T> SelectIndicesBase<'_, &mut [T]>
         // TODO: Safety checks without heap allocation
         
         indices.iter().for_each(|&i| {
+            let i = i.to_usize()
+                .expect("select_indices was given indices that cannot be converted to usize!");
             assert!(i < len);
             assert!(indexset.insert(i));
         });
     }
 }
 
-impl<'a, T: 'a> Iterator for SelectIndicesBase<'a, &'a [T]>
+impl<'a, T: 'a, I: Copy + Clone + PrimInt + ToPrimitive> Iterator for SelectIndicesBase<'a, &'a [T], I>
 {
     type Item = &'a T;
 
@@ -59,7 +65,12 @@ impl<'a, T: 'a> Iterator for SelectIndicesBase<'a, &'a [T]>
         {
             let ind = unsafe { *self.indices.get_unchecked(self.start) };
             self.start += 1;
-            return Some(unsafe { self.data.get_unchecked(ind) });
+            return Some(unsafe { 
+                self.data.get_unchecked(
+                    ind.to_usize()
+                    .expect("select_indices was given indices that cannot be converted to usize!")
+                ) 
+            });
         }
         else {
             return None;
@@ -71,23 +82,28 @@ impl<'a, T: 'a> Iterator for SelectIndicesBase<'a, &'a [T]>
         return (len, Some(len));
     }
 }
-impl<'a, T: 'a> DoubleEndedIterator for SelectIndicesBase<'a, &'a [T]>
+impl<'a, T: 'a, I: Copy + Clone + PrimInt + ToPrimitive> DoubleEndedIterator for SelectIndicesBase<'a, &'a [T], I>
 {
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.start < self.end
         {
             self.end -= 1;
             let ind = unsafe { * self.indices.get_unchecked(self.end) };
-            return Some(unsafe { self.data.get_unchecked(ind) });
+            return Some(unsafe {
+                self.data.get_unchecked(
+                    ind.to_usize()
+                    .expect("select_indices was given indices that cannot be converted to usize!")
+                )
+            });
         }
         else {
             return None;
         }
     }
 }
-impl<'a, T: 'a> ExactSizeIterator for SelectIndicesBase<'a, &'a [T]> {}
+impl<'a, T: 'a, I: Copy + Clone + PrimInt + ToPrimitive> ExactSizeIterator for SelectIndicesBase<'a, &'a [T], I> {}
 
-impl<'a, T: 'a> Iterator for SelectIndicesBase<'a, &'a mut [T]>
+impl<'a, T: 'a, I: Copy + Clone + PrimInt + ToPrimitive> Iterator for SelectIndicesBase<'a, &'a mut [T], I>
 {
     type Item = &'a mut T;
 
@@ -96,7 +112,12 @@ impl<'a, T: 'a> Iterator for SelectIndicesBase<'a, &'a mut [T]>
         {
             let ind = unsafe { *self.indices.get_unchecked(self.start) };
             self.start += 1;
-            return Some(unsafe { &mut *self.data.as_mut_ptr().add(ind) });
+            return Some(unsafe {
+                &mut *self.data.as_mut_ptr().add(
+                    ind.to_usize()
+                    .expect("select_indices was given indices that cannot be converted to usize!")
+                )
+            });
         }
         else {
             return None;
@@ -108,18 +129,23 @@ impl<'a, T: 'a> Iterator for SelectIndicesBase<'a, &'a mut [T]>
         return (len, Some(len));
     }
 }
-impl<'a, T: 'a> DoubleEndedIterator for SelectIndicesBase<'a, &'a mut [T]>
+impl<'a, T: 'a, I: Copy + Clone + PrimInt + ToPrimitive> DoubleEndedIterator for SelectIndicesBase<'a, &'a mut [T], I>
 {
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.start < self.end
         {
             self.end -= 1;
             let ind = unsafe { *self.indices.get_unchecked(self.end) };
-            return Some(unsafe { &mut *self.data.as_mut_ptr().add(ind) });
+            return Some(unsafe {
+                &mut *self.data.as_mut_ptr().add(
+                    ind.to_usize()
+                    .expect("select_indices was given indices that cannot be converted to usize!")
+                )
+            });
         }
         else {
             return None;
         }
     }
 }
-impl<'a, T: 'a> ExactSizeIterator for SelectIndicesBase<'a, &'a mut [T]> {}
\ No newline at end of file
+impl<'a, T: 'a, I: Copy + Clone + PrimInt + ToPrimitive> ExactSizeIterator for SelectIndicesBase<'a, &'a mut [T], I> {}
\ No newline at end of file
